package ap;

import ap.proof.ConstraintSimplifier
import ap.parameters.{Settings, Param}
import ap.util.Debug

object CmdlMain {

  private def printGreeting = {
    println("________       _____")                                 
    println("___  __ \\_________(_)________________________________")
    println("__  /_/ /_  ___/_  /__  __ \\  ___/  _ \\_  ___/_  ___/")
    println("_  ____/_  /   _  / _  / / / /__ /  __/(__  )_(__  )")
    println("/_/     /_/    /_/  /_/ /_/\\___/ \\___//____/ /____/")  
  }
  
  private def printUsage = {
    println("Usage: princess <option>* <inputfile>*")
    println
    println("Options:")
    println("  [+-]printTree             Output the constructed proof tree   (default: -)")
    println("  [+-]simplifyConstraints   Permanently simplify constraints    (default: +)")
    println("  [+-]assert                Enable runtime assertions           (default: +)")
  }
  
  def main(args: Array[String]) : unit = {
    printGreeting
    println
    
    val (settings, inputs) = try { Settings.fromArguments(args) } catch {
      case e : Throwable => {
        println(e.getMessage)
        println
        printUsage
        System exit 1
        null // to make the type of the try-block meaningful
      }
    }
    
    Debug.enableAllAssertions(Param.ASSERTIONS(settings))
    
    try {
      inputs match {
        case Seq() => println("No inputs given, exiting")
        case _ => {
          val simplifier = if (Param.SIMPLIFY_CONSTRAINTS(settings))
                             ConstraintSimplifier.ALL_SIMPLIFIER
                           else
                             ConstraintSimplifier.NO_SIMPLIFIER
          for (filename <- inputs) {
            val timeBefore = System.currentTimeMillis
            
            val prover = new FileProver (new java.io.File (filename),
                                         simplifier, true)
            println
            println("Resulting constraint:")
            println("" + prover.proofTree.closingConstraint)

            if (Param.PRINT_TREE(settings)) {
              println
              println("Proof tree:")
              println(prover.proofTree)
            }
            
            val timeAfter = System.currentTimeMillis
            
            println
            println("" + (timeAfter - timeBefore) + "ms")
          }
        }
      }
    } catch {
      case e : Throwable => {
        println("ERROR: " + e.getMessage)
        System exit 1
      }
    }
  }
}
