package ap.parser;

import ap.terfor.{VariableTerm, ConstantTerm, TermOrder}
import ap.util.FilterIt

object Environment {
  
  abstract class SymKind
  case object Universal extends SymKind
  case object Existential extends SymKind

  abstract class DeclaredSym
  case class Constant(c : ConstantTerm, k : SymKind) extends DeclaredSym
  case class Variable(v : VariableTerm) extends DeclaredSym

  class EnvironmentException(msg : String) extends Exception(msg)

}

class Environment {

  import Environment._
  
  /** The declared symbols */
  private val signature = new scala.collection.mutable.HashMap[String, DeclaredSym]
  
  /** The variables bound at the present point */
  private val context = new scala.collection.mutable.Stack[String]
  
  /** A <code>TermOrder</code> containing all declared constants */
  private var orderVar = TermOrder.EMPTY
  
  def order = orderVar
  
  def lookupSym(name : String) : DeclaredSym = (context lastIndexOf name) match {
    case -1 => (signature get name) match {
      case Some(t) => t
      case None => throw new EnvironmentException("Symbol " + name + " not declared")
    }
    case index => Variable(VariableTerm(context.size - index - 1))
  }
  
  def addConstant(c : ConstantTerm, kind : SymKind) : unit = {
    addSym(c.name, Constant(c, kind))
    // if the constant is universal, make it maximal; otherwise, make it minimal
    orderVar = kind match {
      case Universal => orderVar.extend(c, Set.empty)
      case Existential => orderVar.extend(c, orderVar.orderedConstants)
    }
  }
  
  def universalConstants : Set[ConstantTerm] = {
    val predicate : DeclaredSym => boolean = {
      case Constant(_, Universal) => true
      case _ => false
    }
    Set.empty ++ (for (Constant(c, _) <- FilterIt(signature.values, predicate))
                  yield c)
  }
  
  private def addSym(name : String, t : DeclaredSym) : unit =
    if (signature contains name)
      throw new EnvironmentException("Symbol " + name + " is already declared")
    else
      signature += (name -> t)
  
  def pushVar(name : String) : unit = (context += name)

  def popVar : unit =    
    if (context isEmpty)
      throw new EnvironmentException("Trying to pop a non-existing variable")
    else
      context pop
  
}
