package ap.proof;

import ap.terfor.Quantifier
import ap.proof.goal.{Task, Goal}
import ap.terfor.{Formula, Conjunction, ConstantTerm, TermOrder}
import ap.terfor.substitutions.Substitution

abstract class ProofTreeFactory {

  def and(subtrees : Seq[ProofTree], order : TermOrder) : ProofTree
  
  def quantify(subtree : ProofTree,
               quan : Quantifier,
               quantifiedConstants : Set[ConstantTerm],
               order : TermOrder,
               subtreeOrder : TermOrder) : ProofTree
  
  def substitute(subtree : ProofTree,
                 subst : Substitution,
                 order : TermOrder,
                 subtreeOrder : TermOrder) : ProofTree
               
  def weaken(subtree : ProofTree,
             disjunct : Formula,
             order : TermOrder) : ProofTree
                 
  //////////////////////////////////////////////////////////////////////////////
  
  def updateGoal(updatedFacts : Conjunction,
                 updatedElimConstants : Set[ConstantTerm],
                 updatedOrder : TermOrder,
                 newTasks : Iterable[Task],
                 goal : Goal) : Goal

  def updateGoal(updatedElimConstants : Set[ConstantTerm],
                 updatedOrder : TermOrder,
                 newTasks : Iterable[Task],
                 goal : Goal) : Goal =
    updateGoal(goal.facts, updatedElimConstants, updatedOrder, newTasks, goal)

  def updateGoal(updatedFacts : Conjunction,
                 newTasks : Iterable[Task],
                 goal : Goal) : Goal =
    updateGoal(updatedFacts, goal.eliminatedConstants, goal.order, newTasks, goal)

  def updateGoal(updatedFacts : Conjunction, goal : Goal) : Goal =
    updateGoal(updatedFacts, goal.eliminatedConstants, goal.order, List(), goal)

  def updateGoal(newTasks : Iterable[Task], goal : Goal) : Goal =
    updateGoal(goal.facts, goal.eliminatedConstants, goal.order, newTasks, goal)

  def updateGoal(goal : Goal) : Goal =
    updateGoal(goal.facts, goal.eliminatedConstants, goal.order, List(), goal)

}
