package ap.proof;

import ap.terfor.Quantifier
import ap.proof.goal.{Task, Goal}
import ap.terfor.{Formula, Conjunction, ConstantTerm, TermOrder}
import ap.terfor.substitutions.Substitution

class SimpleProofTreeFactory(removeTask : Boolean,
                             simplifier : ConstraintSimplifier) extends ProofTreeFactory {

  def and(subtrees : Seq[ProofTree], order : TermOrder) : ProofTree =
    AndTree(subtrees, order, simplifier)
  
  def quantify(subtree : ProofTree,
               quan : Quantifier,
               quantifiedConstants : Set[ConstantTerm],
               order : TermOrder,
               subtreeOrder : TermOrder) : ProofTree =
    QuantifiedTree(subtree, quan, quantifiedConstants, order, subtreeOrder, simplifier)
  
  def substitute(subtree : ProofTree,
                 subst : Substitution,
                 order : TermOrder,
                 subtreeOrder : TermOrder) : ProofTree =
    SubstTree(subtree, subst, order, subtreeOrder, simplifier)
               
  def weaken(subtree : ProofTree,
             disjunct : Formula,
             order : TermOrder) : ProofTree =
    WeakenTree(subtree, disjunct, order)
                 
  //////////////////////////////////////////////////////////////////////////////
  
  def updateGoal(updatedFacts : Conjunction,
                 updatedElimConstants : Set[ConstantTerm],
                 updatedOrder : TermOrder,
                 newTasks : Iterable[Task],
                 goal : Goal) : Goal = {
    val updatedTasks = (if (removeTask)
                          goal.tasks.removeFirst
                        else
                          goal.tasks) ++ newTasks
    Goal(updatedFacts, updatedTasks, goal.age + 1, updatedElimConstants, updatedOrder)
  }
  
}
