package ap.proof.goal;

/**
 * Trait for the future tasks that are stored in a proof goal. Each task is 
 * given a <code>priority</code> that is based on the age (to ensure fairness)
 * and on other criteria to work at different tasks in a meaningful order.
 * Tasks with smaller <code>priority</code> are supposed to be handled first.
 * Tasks are considered as mappings from proof goals to proof trees, given
 * a continuation function that tells what should be done with the results
 * (premiss goals) of performing the task. 
 */
trait Task {

  val priority : Int
  
  def apply(goal : Goal, ptf : ProofTreeFactory) : ProofTree
  
}
